@@ -21,13 +21,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  *
- * $Id: scim_hangul_imengine.h,v 1.3 2006/01/17 02:17:01 suzhe Exp $
+ * $Id: scim_hangul_imengine.h,v 1.16 2006/12/02 02:27:06 hwanjin Exp $
  */
 
 #if !defined (__SCIM_HANGUL_IMENGINE_H)
 #define __SCIM_HANGUL_IMENGINE_H
 
-#include "hangul.h"
+#include <hangul.h>
 
 using namespace scim;
 
@@ -45,38 +45,35 @@ enum HangulOutputMode
     OUTPUT_MODE_JAMO_EXT = 1 << 2
 };
 
-struct HangulCombination 
-{
-  uint32  key;
-  ucs4_t  code;
-};
-
-struct HangulFactoryData
-{
-    const char              *uuid;
-    const char              *name;
-
-    HangulKeyboardType       keyboard_type;
-};
-
 class HangulFactory : public IMEngineFactoryBase
 {
     String                   m_uuid;
     String                   m_name;
 
-    HangulKeyboardType       m_keyboard_type;
+    ConfigPointer            m_config;
+
+    String                   m_keyboard_layout;
 
     bool                     m_always_use_jamo;
 
     bool                     m_show_candidate_comment;
+    bool                     m_lookup_table_vertical;
+    bool                     m_use_ascii_mode;
+    bool                     m_commit_by_word;
+    bool                     m_hanja_mode;
 
-    KeyEventList             m_trigger_keys;
-    KeyEventList             m_hangul_hanja_keys;
+    KeyEventList             m_hangul_keys;
+    KeyEventList             m_hanja_keys;
+    KeyEventList             m_hanja_mode_keys;
+
+    Connection               m_reload_signal_connection;
+
+    HanjaTable*              m_hanja_table;
 
     friend class HangulInstance;
 
 public:
-    HangulFactory (const ConfigPointer &config, const HangulFactoryData &data);
+    HangulFactory (const ConfigPointer &config);
 
     virtual ~HangulFactory ();
 
@@ -88,6 +85,9 @@ public:
     virtual String      get_icon_file () const;
 
     virtual IMEngineInstancePointer create_instance (const String& encoding, int id = -1);
+
+private:
+    void reload_config (const ConfigPointer &config);
 };
 
 class HangulInstance : public IMEngineInstanceBase
@@ -96,26 +96,16 @@ class HangulInstance : public IMEngineInstanceBase
 
     CommonLookupTable    m_lookup_table;
     std::vector<String>  m_candidate_comments;
+    WideString           m_preedit;
+    WideString           m_surrounding_text;
 
     KeyEvent             m_prev_key;
 
     HangulInputContext  *m_hic;
 
-    int                  m_index;         /* stack index */
-    ucs4_t               m_stack     [12];
-
-    int                  m_lindex;        /* leading consonant index */
-    int                  m_vindex;        /* vowel index */
-    int                  m_tindex;        /* trailing consonant index */
-    ucs4_t               m_choseong  [4];
-    ucs4_t               m_jungseong [4];
-    ucs4_t               m_jongseong [4];
-
-    int                  m_input_mode;
+    bool                 m_hangul_mode;
     int                  m_output_mode;
 
-    bool (HangulInstance::*m_hangul_composer) (const KeyEvent &key);
-
 public:
     HangulInstance (HangulFactory *factory,
                     const String  &encoding,
@@ -130,6 +120,7 @@ public:
     virtual void lookup_table_page_up ();
     virtual void lookup_table_page_down ();
     virtual void reset ();
+    virtual void flush ();
     virtual void focus_in ();
     virtual void focus_out ();
     virtual void trigger_property (const String &property);
@@ -139,33 +130,61 @@ private:
         return (key.code == SCIM_KEY_BackSpace);
     }
 
-    bool is_hangul_hanja_key (const KeyEvent &key) const {
-        return match_key_event (m_factory->m_hangul_hanja_keys, key);
+    bool is_hangul_key (const KeyEvent &key) const {
+        return match_key_event (m_factory->m_hangul_keys, key);
+    }
+
+    bool is_hanja_key (const KeyEvent &key) const {
+        return match_key_event (m_factory->m_hanja_keys, key);
+    }
+
+    bool is_hanja_mode_key (const KeyEvent &key) const {
+        return match_key_event (m_factory->m_hanja_mode_keys, key);
     }
 
     /* preedit string */
     WideString get_preedit_string () {
-        WideString wstr;
-        const wchar_t *wcs = hangul_ic_get_preedit_string(m_hic);
-        while (*wcs != L'\0')
-            wstr.push_back (*wcs++);
+        WideString wstr = m_preedit;
+        const ucschar *str = hangul_ic_get_preedit_string(m_hic);
+        while (*str != 0)
+            wstr.push_back (*str++);
         return wstr;
     }
 
     WideString get_commit_string () {
         WideString wstr;
-        const wchar_t *wcs = hangul_ic_get_commit_string(m_hic);
-        while (*wcs != L'\0')
-            wstr.push_back (*wcs++);
+        const ucschar *str = hangul_ic_get_commit_string(m_hic);
+        while (*str != L'\0')
+            wstr.push_back (*str++);
         return wstr;
     }
 
     void hangul_update_preedit_string ();
 
+    bool   use_ascii_mode() {
+        return m_factory->m_use_ascii_mode;
+    }
+
+    bool   is_hangul_mode() {
+        return m_hangul_mode;
+    }
+
+    bool   is_hanja_mode() {
+        return m_factory->m_hanja_mode;
+    }
+
+    void   toggle_hangul_mode();
+    void   toggle_hanja_mode();
+    void   change_keyboard_layout(const String &layout);
+
+    /* property handling */
+    void   register_all_properties();
+
     /* aux string */
     void hangul_update_aux_string ();
 
     /* candidate functions */
+    String get_candidate_string();
     void   update_candidates ();
     void   delete_candidates ();
 
